﻿#target bridge

var  f = File.openDialog('Choice PSD file', '*.psd');

if (f) {
	
	var bd = new BitmapData(f), 
		out = bd.clone(),
		
		w = bd.width, 
		h = bd.height,
		
		nodes = [], 
		distances = [], 
		k = [],
		
		window = new Window('window', '2D Gradient'),
		pb = window.add('progressbar', {x:9, y:9, width:400, height:20}, 0, w * 4);
	
	window.show();

	seekNodes();
	calkDistances();
	calkK();
	fillGradient();
	
	window.hide();

	outFile = new File(f.fsName + '_2DG.jpeg');
	outFile.remove();
	out.exportTo(outFile, 100);
}


function seekNodes() {
	var w = bd.width, h = bd.height;
	
	for (var x = 0; x < w; x++) {
		for (var y = 0; y < h; y++ ) {
			
			var color = new Color(bd.getPixel32(x,y));
			
			if (color.red == 255 && color.green == 0 && color.blue == 255) {
				continue;
			}
			var node = {
				x: x,
				y: y,
				red: color.red,
				green: color.green,
				blue: color.blue
			}
			nodes.push(node);
		}
		pb.value++;
	}
}

function calkDistances(){
	
	for (var x = 0; x < w; x++) {
		distances[x] = [];
		
		for (var y = 0; y < h; y++ ) {
			distances[x][y] = [];
			var sum = 0;
			
			for (var i = 0; i < nodes.length; i++) {
				var distance = getDistance({x: x, y: y}, nodes[i]);
				
				if (distance == 0) { // this is node
					distances[x][y] = null;
					break;
				}
				distances[x][y][i] = distance;
				sum += distance;
				
			}
			distances[x][y] && distances[x][y].sum = sum;
		}
		pb.value++;
	}
}

function calkK() {
	
	for (var x = 0; x < w; x++) {
		k[x] = [];
		
		for (var y = 0; y < h; y++ ) {
			k[x][y] = [];
			var sum = 0;
			
			if (distances[x][y] === null) continue;
			
			for (var i = 0; i < nodes.length; i++) {
				
				var kValue = distances[x][y].sum / distances[x][y][i] / nodes.length;
				kValue = Math.pow(kValue, Math.E);
				
				k[x][y][i] = kValue
				sum += kValue;
			}
			
			k[x][y].sum = sum;
		}
		pb.value++;
	}
}

function fillGradient(){
	
	for (var x = 0; x < w; x++) {
		for (var y = 0; y < h; y++ ) {
			
			if (distances[x][y] === null) continue;
			
			var red = 0, green = 0, blue = 0;
			var sumK = k[x][y].sum;
			
			for (var i = 0; i < nodes.length; i ++ ) {
				red +=   nodes[i].red   * k[x][y][i];
				green += nodes[i].green * k[x][y][i];
				blue +=  nodes[i].blue  * k[x][y][i];
			}
			red =   red   / sumK;
			green = green / sumK;
			blue =  blue  / sumK;
			
			out.setPixel32(x, y, new Color(red, green, blue));
		}
		pb.value++;
	}
}

function getDistance(point1, point2){
	
	return Math.sqrt(Math.pow(point1.x - point2.x, 2) + Math.pow(point1.y - point2.y, 2));
}